@@ -317,19 +317,11 @@ public:
 	/** fts_t destructor. */
 	~fts_t();
 
-	/** Mutex protecting bg_threads* and fts_add_wq. */
-	ib_mutex_t	bg_threads_mutex;
-
-	/** Whether the ADDED table record sync-ed after
-	crash recovery; protected by bg_threads_mutex */
+	/** Whether the ADDED table record sync-ed after crash recovery */
 	unsigned	added_synced:1;
-	/** Whether the table holds dict_sys->mutex;
-	protected by bg_threads_mutex */
+	/** Whether the table holds dict_sys->mutex */
 	unsigned	dict_locked:1;
 
-	/** Number of background threads accessing this table. */
-	ulint		bg_threads;
-
 	/** Work queue for scheduling jobs for the FTS 'Add' thread, or NULL
 	if the thread has not yet been created. Each work item is a
 	fts_trx_doc_ids_t*. */
@@ -735,12 +727,9 @@ fts_savepoint_rollback_last_stmt(
 /*=============================*/
 	trx_t*		trx);			/*!< in: transaction */
 
-/***********************************************************************//**
-Drop all orphaned FTS auxiliary tables, those that don't have a parent
+/** Drop all orphaned FTS auxiliary tables, those that don't have a parent
 table or FTS index defined on them. */
-void
-fts_drop_orphaned_tables(void);
-/*==========================*/
+void fts_drop_orphaned_tables();
 
 /** Run SYNC on the table, i.e., write out data from the cache to the
 FTS auxiliary INDEX table and clear the cache at the end.
@@ -774,15 +763,6 @@ fts_init_doc_id(
 /*============*/
 	const dict_table_t*		table);	/*!< in: table */
 
-/* Get parent table name if it's a fts aux table
-@param[in]	aux_table_name	aux table name
-@param[in]	aux_table_len	aux table length
-@return parent table name, or NULL */
-char*
-fts_get_parent_table_name(
-	const char*	aux_table_name,
-	ulint		aux_table_len);
-
 /******************************************************************//**
 compare two character string according to their charset. */
 extern
@@ -983,4 +963,20 @@ fts_trx_t*
 fts_trx_create(
 	trx_t*  trx);
 
+/** Clear all fts resources when there is no internal DOC_ID
+and there are no new fts index to add.
+@param[in,out]  table   table  where fts is to be freed
+@param[in]      trx     transaction to drop all fts tables */
+void fts_clear_all(dict_table_t *table, trx_t *trx);
+
+/** Check whether the given name is fts auxiliary table
+and fetch the parent table id and index id
+@param[in]	name		table name
+@param[in,out]	table_id	parent table id
+@param[in,out]	index_id	index id
+@return true if it is auxilary table */
+bool fts_check_aux_table(const char *name,
+                         table_id_t *table_id,
+                         index_id_t *index_id);
+
 #endif /*!< fts0fts.h */
